from utils import randbool  # импортирование своего метода
from utils import randcell, randcell2
import os


# -------- SETTINGS -----------
TREE_BONUS = 100 # кол-во очков за потушенный пожар
UPDATE_COST = 500 # стоимосто апгрейда
UPDATE_WEIGHT = 1 # кол-во апгрейда
LIFE_COST = 1000 # стоимость покупки жизни
LIFE_WEIGHT = 10 # колво покупаемых жизней



os.system('') # см. в мар.ру
 # см. в мар.ру
BLACK = '\033[30m'
RED = '\033[91m'
GREEN = '\033[92m'
YELLOW = '\033[33m'
CYAN = '\033[36m'
WHITE = '\033[37m'
BLD = '\033[1m'
UNDR = '\033[4m'
RST = '\033[0m'

# 0 - поле
# 1 - дерево
# 2 - река
# 3 - госпиталь
# 4 - upgrade shop
# 5 - огонь

#CELL_TYPES = "✅🌲🌊🏥💵🔥"
#CELL_TYPES = "░♠▓⌂$ﬓ"
CELL_TYPES = (  BLACK+'█'+RST,
                GREEN+'♠'+RST,
                CYAN+'█'+RST,
                BLD+WHITE+'⌂'+RST,
                YELLOW+'$'+RST,
                RED+'ﬓ'+RST)

class Map:

    def __init__(self, w, h):  # Принимает ширину и высоту карты
        # Изначально везде будут поля
        self.w = w
        self.h = h
        self.cells = [[0 for i in range(w)] for j in range(h)] # создаем карту
        self.generate_forest(5, 20) # запускаем создание леса
        self.generate_river(10) # запускаем создание озера №1 размером в n блоков
        self.generate_river(30) # запускаем создание озера №2
        self.generate_upgrade_shop() # запускаем создание магазина
        self.generate_hospital() # запускаем создание больницы

    def check_bounds(self, x, y):  # Проверка выхода за границы
        if x < 0 or y < 0 or x >= self.h or y >= self.w:
            return False
        return True

    def print_map(self, helico, clouds):  # Вывод карты
        print('\t'+('█' * (self.w + 2)))  # вершняя рамка
        for ri in range(self.h):
            print('\t█', end="")  # левая рамка
            for ci in range(self.w):
                cell = self.cells[ri][ci]
                if clouds.cells[ri][ci] == 1:
                    print(BLD+WHITE+"Ҩ"+RST, end="") # рисуем облока
                elif clouds.cells[ri][ci] == 2:
                    print(YELLOW+"Ϟ"+RST, end="") # рисуем грозу
                elif helico.x == ri and helico.y == ci:
                    if helico.tank >= 1:
                        print(UNDR+CYAN+"₫"+RST, end="") # рисуем заправленный вертолет
                    else:
                        print(UNDR+YELLOW+"₫"+RST, end="") # рисуем пустой вертолет
                elif cell >= 0 and cell < len(CELL_TYPES):
                    print(CELL_TYPES[cell], end="") # рисуем поле и 
                                                    # элимент равный
                                                    # индексу из CELL_TYPES
            print('█')  # правая рамка
        print('\t'+('█' * (self.w + 2)))  # нижняя рамка

    def generate_upgrade_shop(self): # создаем магазин на карте
        c = randcell(self.w, self.h)
        cx, cy = c[0], c[1]
        self.cells[cx][cy] = 4

    def generate_hospital(self): # создаем больницу на карте
        c = randcell(self.w, self.h)
        cx, cy = c[0], c[1]
        if self.cells[cx][cy] == 4: # чтобы не поставить в том же месте где и магаз.
            self.generate_hospital()
        else:
            self.cells[cx][cy] = 3

    def generate_river(self, l):  # генератор располж воды
        rc = randcell(self.w, self.h)
        rx, ry = rc[0], rc[1]
        while l >= 0:
            rc2 = randcell2(rx, ry)
            rx2, ry2 = rc2[0], rc2[1]
            if self.check_bounds(rx, ry) and self.check_bounds(rx2, ry2):
                self.cells[rx2][ry2] = 2
                rx, ry = rx2, ry2
                l -= 1

    def generate_forest(self, r, mxr):  # Генератор лесов
        for ri in range(self.h):
            for ci in range(self.w):
                if randbool(r, mxr):
                    self.cells[ri][ci] = 1

    def generate_tree(self):  # генерацию дерева (одиночного) в поле
        c = randcell(self.w, self.h)
        cx, cy = c[0], c[1]  # Координаты случайного дерева
        if self.check_bounds(cx, cy) and self.cells[cx][cy] == 0: # что место ни чем не занято
            self.cells[cx][cy] = 1

    def add_fire(self):  # Генерации пожара
        c = randcell(self.w, self.h)
        cx, cy = c[0], c[1]
        if self.cells[cx][cy] == 1:
            self.cells[cx][cy] = 5

    def update_fires(self): # обновление леса для нового кадра
        for ri in range(self.h):
            for ci in range(self.w):
                cell = self.cells[ri][ci]
                if cell == 5:
                    self.cells[ri][ci] = 0
        for i in range(5):
            self.add_fire()

    def process_hilcopter(self, helico, clouds): # действия вертолета
        c = self.cells[helico.x][helico.y]
        d = clouds.cells[helico.x][helico.y]
        if c == 2: # заправка
            helico.tank = helico.mxtank
        if c == 5 and helico.tank > 0:  # тушение пожара
            helico.tank -= 1 # опустошение бака
            self.cells[helico.x][helico.y] = 1
            helico.score += TREE_BONUS # начисление очков
        if c == 4 and helico.score >= UPDATE_COST: # апгрейд бака
            helico.mxtank += UPDATE_WEIGHT
            helico.score -= UPDATE_COST
        if c == 3 and helico.score >= LIFE_COST: # лечение
            helico.lives += LIFE_WEIGHT
            helico.score -= LIFE_COST
        if d == 2: # взаимодействие с грозой
            helico.lives -= 1
            if helico.lives == 0:
                helico.game_over()
    
    def export_data(self): # данные здя загрузки в файл
        return {"cells": self.cells}

    def import_data(self, data): # восстонавливаем карту из файла
        self.cells = data['cells'] or [[0 for i in range(self.w)] for j in range(self.h)]
